"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SPECIAL_HEADERS = exports.HEADER_STATE = exports.MINOR = exports.MAJOR = exports.HTAB_SP_VCHAR_OBS_TEXT = exports.QUOTED_STRING = exports.CONNECTION_TOKEN_CHARS = exports.HEADER_CHARS = exports.TOKEN = exports.HEX = exports.URL_CHAR = exports.USERINFO_CHARS = exports.MARK = exports.ALPHANUM = exports.NUM = exports.HEX_MAP = exports.NUM_MAP = exports.ALPHA = exports.FINISH = exports.STATUSES_HTTP = exports.STATUSES = exports.H_METHOD_MAP = exports.METHOD_MAP = exports.METHODS_RTSP = exports.METHODS_ICE = exports.METHODS_HTTP = exports.METHODS = exports.LENIENT_FLAGS = exports.FLAGS = exports.TYPE = exports.ERROR = void 0;
const utils_1 = require("./utils");
// C headers
var ERROR;
(function (ERROR) {
    ERROR[ERROR["OK"] = 0] = "OK";
    ERROR[ERROR["INTERNAL"] = 1] = "INTERNAL";
    ERROR[ERROR["STRICT"] = 2] = "STRICT";
    ERROR[ERROR["CR_EXPECTED"] = 25] = "CR_EXPECTED";
    ERROR[ERROR["LF_EXPECTED"] = 3] = "LF_EXPECTED";
    ERROR[ERROR["UNEXPECTED_CONTENT_LENGTH"] = 4] = "UNEXPECTED_CONTENT_LENGTH";
    ERROR[ERROR["UNEXPECTED_SPACE"] = 30] = "UNEXPECTED_SPACE";
    ERROR[ERROR["CLOSED_CONNECTION"] = 5] = "CLOSED_CONNECTION";
    ERROR[ERROR["INVALID_METHOD"] = 6] = "INVALID_METHOD";
    ERROR[ERROR["INVALID_URL"] = 7] = "INVALID_URL";
    ERROR[ERROR["INVALID_CONSTANT"] = 8] = "INVALID_CONSTANT";
    ERROR[ERROR["INVALID_VERSION"] = 9] = "INVALID_VERSION";
    ERROR[ERROR["INVALID_HEADER_TOKEN"] = 10] = "INVALID_HEADER_TOKEN";
    ERROR[ERROR["INVALID_CONTENT_LENGTH"] = 11] = "INVALID_CONTENT_LENGTH";
    ERROR[ERROR["INVALID_CHUNK_SIZE"] = 12] = "INVALID_CHUNK_SIZE";
    ERROR[ERROR["INVALID_STATUS"] = 13] = "INVALID_STATUS";
    ERROR[ERROR["INVALID_EOF_STATE"] = 14] = "INVALID_EOF_STATE";
    ERROR[ERROR["INVALID_TRANSFER_ENCODING"] = 15] = "INVALID_TRANSFER_ENCODING";
    ERROR[ERROR["CB_MESSAGE_BEGIN"] = 16] = "CB_MESSAGE_BEGIN";
    ERROR[ERROR["CB_HEADERS_COMPLETE"] = 17] = "CB_HEADERS_COMPLETE";
    ERROR[ERROR["CB_MESSAGE_COMPLETE"] = 18] = "CB_MESSAGE_COMPLETE";
    ERROR[ERROR["CB_CHUNK_HEADER"] = 19] = "CB_CHUNK_HEADER";
    ERROR[ERROR["CB_CHUNK_COMPLETE"] = 20] = "CB_CHUNK_COMPLETE";
    ERROR[ERROR["PAUSED"] = 21] = "PAUSED";
    ERROR[ERROR["PAUSED_UPGRADE"] = 22] = "PAUSED_UPGRADE";
    ERROR[ERROR["PAUSED_H2_UPGRADE"] = 23] = "PAUSED_H2_UPGRADE";
    ERROR[ERROR["USER"] = 24] = "USER";
    ERROR[ERROR["CB_URL_COMPLETE"] = 26] = "CB_URL_COMPLETE";
    ERROR[ERROR["CB_STATUS_COMPLETE"] = 27] = "CB_STATUS_COMPLETE";
    ERROR[ERROR["CB_METHOD_COMPLETE"] = 32] = "CB_METHOD_COMPLETE";
    ERROR[ERROR["CB_VERSION_COMPLETE"] = 33] = "CB_VERSION_COMPLETE";
    ERROR[ERROR["CB_HEADER_FIELD_COMPLETE"] = 28] = "CB_HEADER_FIELD_COMPLETE";
    ERROR[ERROR["CB_HEADER_VALUE_COMPLETE"] = 29] = "CB_HEADER_VALUE_COMPLETE";
    ERROR[ERROR["CB_CHUNK_EXTENSION_NAME_COMPLETE"] = 34] = "CB_CHUNK_EXTENSION_NAME_COMPLETE";
    ERROR[ERROR["CB_CHUNK_EXTENSION_VALUE_COMPLETE"] = 35] = "CB_CHUNK_EXTENSION_VALUE_COMPLETE";
    ERROR[ERROR["CB_RESET"] = 31] = "CB_RESET";
})(ERROR = exports.ERROR || (exports.ERROR = {}));
var TYPE;
(function (TYPE) {
    TYPE[TYPE["BOTH"] = 0] = "BOTH";
    TYPE[TYPE["REQUEST"] = 1] = "REQUEST";
    TYPE[TYPE["RESPONSE"] = 2] = "RESPONSE";
})(TYPE = exports.TYPE || (exports.TYPE = {}));
var FLAGS;
(function (FLAGS) {
    FLAGS[FLAGS["CONNECTION_KEEP_ALIVE"] = 1] = "CONNECTION_KEEP_ALIVE";
    FLAGS[FLAGS["CONNECTION_CLOSE"] = 2] = "CONNECTION_CLOSE";
    FLAGS[FLAGS["CONNECTION_UPGRADE"] = 4] = "CONNECTION_UPGRADE";
    FLAGS[FLAGS["CHUNKED"] = 8] = "CHUNKED";
    FLAGS[FLAGS["UPGRADE"] = 16] = "UPGRADE";
    FLAGS[FLAGS["CONTENT_LENGTH"] = 32] = "CONTENT_LENGTH";
    FLAGS[FLAGS["SKIPBODY"] = 64] = "SKIPBODY";
    FLAGS[FLAGS["TRAILING"] = 128] = "TRAILING";
    // 1 << 8 is unused
    FLAGS[FLAGS["TRANSFER_ENCODING"] = 512] = "TRANSFER_ENCODING";
})(FLAGS = exports.FLAGS || (exports.FLAGS = {}));
var LENIENT_FLAGS;
(function (LENIENT_FLAGS) {
    LENIENT_FLAGS[LENIENT_FLAGS["HEADERS"] = 1] = "HEADERS";
    LENIENT_FLAGS[LENIENT_FLAGS["CHUNKED_LENGTH"] = 2] = "CHUNKED_LENGTH";
    LENIENT_FLAGS[LENIENT_FLAGS["KEEP_ALIVE"] = 4] = "KEEP_ALIVE";
    LENIENT_FLAGS[LENIENT_FLAGS["TRANSFER_ENCODING"] = 8] = "TRANSFER_ENCODING";
    LENIENT_FLAGS[LENIENT_FLAGS["VERSION"] = 16] = "VERSION";
    LENIENT_FLAGS[LENIENT_FLAGS["DATA_AFTER_CLOSE"] = 32] = "DATA_AFTER_CLOSE";
    LENIENT_FLAGS[LENIENT_FLAGS["OPTIONAL_LF_AFTER_CR"] = 64] = "OPTIONAL_LF_AFTER_CR";
    LENIENT_FLAGS[LENIENT_FLAGS["OPTIONAL_CRLF_AFTER_CHUNK"] = 128] = "OPTIONAL_CRLF_AFTER_CHUNK";
    LENIENT_FLAGS[LENIENT_FLAGS["OPTIONAL_CR_BEFORE_LF"] = 256] = "OPTIONAL_CR_BEFORE_LF";
    LENIENT_FLAGS[LENIENT_FLAGS["SPACES_AFTER_CHUNK_SIZE"] = 512] = "SPACES_AFTER_CHUNK_SIZE";
})(LENIENT_FLAGS = exports.LENIENT_FLAGS || (exports.LENIENT_FLAGS = {}));
var METHODS;
(function (METHODS) {
    METHODS[METHODS["DELETE"] = 0] = "DELETE";
    METHODS[METHODS["GET"] = 1] = "GET";
    METHODS[METHODS["HEAD"] = 2] = "HEAD";
    METHODS[METHODS["POST"] = 3] = "POST";
    METHODS[METHODS["PUT"] = 4] = "PUT";
    /* pathological */
    METHODS[METHODS["CONNECT"] = 5] = "CONNECT";
    METHODS[METHODS["OPTIONS"] = 6] = "OPTIONS";
    METHODS[METHODS["TRACE"] = 7] = "TRACE";
    /* WebDAV */
    METHODS[METHODS["COPY"] = 8] = "COPY";
    METHODS[METHODS["LOCK"] = 9] = "LOCK";
    METHODS[METHODS["MKCOL"] = 10] = "MKCOL";
    METHODS[METHODS["MOVE"] = 11] = "MOVE";
    METHODS[METHODS["PROPFIND"] = 12] = "PROPFIND";
    METHODS[METHODS["PROPPATCH"] = 13] = "PROPPATCH";
    METHODS[METHODS["SEARCH"] = 14] = "SEARCH";
    METHODS[METHODS["UNLOCK"] = 15] = "UNLOCK";
    METHODS[METHODS["BIND"] = 16] = "BIND";
    METHODS[METHODS["REBIND"] = 17] = "REBIND";
    METHODS[METHODS["UNBIND"] = 18] = "UNBIND";
    METHODS[METHODS["ACL"] = 19] = "ACL";
    /* subversion */
    METHODS[METHODS["REPORT"] = 20] = "REPORT";
    METHODS[METHODS["MKACTIVITY"] = 21] = "MKACTIVITY";
    METHODS[METHODS["CHECKOUT"] = 22] = "CHECKOUT";
    METHODS[METHODS["MERGE"] = 23] = "MERGE";
    /* upnp */
    METHODS[METHODS["M-SEARCH"] = 24] = "M-SEARCH";
    METHODS[METHODS["NOTIFY"] = 25] = "NOTIFY";
    METHODS[METHODS["SUBSCRIBE"] = 26] = "SUBSCRIBE";
    METHODS[METHODS["UNSUBSCRIBE"] = 27] = "UNSUBSCRIBE";
    /* RFC-5789 */
    METHODS[METHODS["PATCH"] = 28] = "PATCH";
    METHODS[METHODS["PURGE"] = 29] = "PURGE";
    /* CalDAV */
    METHODS[METHODS["MKCALENDAR"] = 30] = "MKCALENDAR";
    /* RFC-2068, section 19.6.1.2 */
    METHODS[METHODS["LINK"] = 31] = "LINK";
    METHODS[METHODS["UNLINK"] = 32] = "UNLINK";
    /* icecast */
    METHODS[METHODS["SOURCE"] = 33] = "SOURCE";
    /* RFC-7540, section 11.6 */
    METHODS[METHODS["PRI"] = 34] = "PRI";
    /* RFC-2326 RTSP */
    METHODS[METHODS["DESCRIBE"] = 35] = "DESCRIBE";
    METHODS[METHODS["ANNOUNCE"] = 36] = "ANNOUNCE";
    METHODS[METHODS["SETUP"] = 37] = "SETUP";
    METHODS[METHODS["PLAY"] = 38] = "PLAY";
    METHODS[METHODS["PAUSE"] = 39] = "PAUSE";
    METHODS[METHODS["TEARDOWN"] = 40] = "TEARDOWN";
    METHODS[METHODS["GET_PARAMETER"] = 41] = "GET_PARAMETER";
    METHODS[METHODS["SET_PARAMETER"] = 42] = "SET_PARAMETER";
    METHODS[METHODS["REDIRECT"] = 43] = "REDIRECT";
    METHODS[METHODS["RECORD"] = 44] = "RECORD";
    /* RAOP */
    METHODS[METHODS["FLUSH"] = 45] = "FLUSH";
})(METHODS = exports.METHODS || (exports.METHODS = {}));
exports.METHODS_HTTP = [
    METHODS.DELETE,
    METHODS.GET,
    METHODS.HEAD,
    METHODS.POST,
    METHODS.PUT,
    METHODS.CONNECT,
    METHODS.OPTIONS,
    METHODS.TRACE,
    METHODS.COPY,
    METHODS.LOCK,
    METHODS.MKCOL,
    METHODS.MOVE,
    METHODS.PROPFIND,
    METHODS.PROPPATCH,
    METHODS.SEARCH,
    METHODS.UNLOCK,
    METHODS.BIND,
    METHODS.REBIND,
    METHODS.UNBIND,
    METHODS.ACL,
    METHODS.REPORT,
    METHODS.MKACTIVITY,
    METHODS.CHECKOUT,
    METHODS.MERGE,
    METHODS['M-SEARCH'],
    METHODS.NOTIFY,
    METHODS.SUBSCRIBE,
    METHODS.UNSUBSCRIBE,
    METHODS.PATCH,
    METHODS.PURGE,
    METHODS.MKCALENDAR,
    METHODS.LINK,
    METHODS.UNLINK,
    METHODS.PRI,
    // TODO(indutny): should we allow it with HTTP?
    METHODS.SOURCE,
];
exports.METHODS_ICE = [
    METHODS.SOURCE,
];
exports.METHODS_RTSP = [
    METHODS.OPTIONS,
    METHODS.DESCRIBE,
    METHODS.ANNOUNCE,
    METHODS.SETUP,
    METHODS.PLAY,
    METHODS.PAUSE,
    METHODS.TEARDOWN,
    METHODS.GET_PARAMETER,
    METHODS.SET_PARAMETER,
    METHODS.REDIRECT,
    METHODS.RECORD,
    METHODS.FLUSH,
    // For AirPlay
    METHODS.GET,
    METHODS.POST,
];
exports.METHOD_MAP = (0, utils_1.enumToMap)(METHODS);
exports.H_METHOD_MAP = {};
for (const key of Object.keys(exports.METHOD_MAP)) {
    if (/^H/.test(key)) {
        exports.H_METHOD_MAP[key] = exports.METHOD_MAP[key];
    }
}
var STATUSES;
(function (STATUSES) {
    STATUSES[STATUSES["CONTINUE"] = 100] = "CONTINUE";
    STATUSES[STATUSES["SWITCHING_PROTOCOLS"] = 101] = "SWITCHING_PROTOCOLS";
    STATUSES[STATUSES["PROCESSING"] = 102] = "PROCESSING";
    STATUSES[STATUSES["EARLY_HINTS"] = 103] = "EARLY_HINTS";
    STATUSES[STATUSES["RESPONSE_IS_STALE"] = 110] = "RESPONSE_IS_STALE";
    STATUSES[STATUSES["REVALIDATION_FAILED"] = 111] = "REVALIDATION_FAILED";
    STATUSES[STATUSES["DISCONNECTED_OPERATION"] = 112] = "DISCONNECTED_OPERATION";
    STATUSES[STATUSES["HEURISTIC_EXPIRATION"] = 113] = "HEURISTIC_EXPIRATION";
    STATUSES[STATUSES["MISCELLANEOUS_WARNING"] = 199] = "MISCELLANEOUS_WARNING";
    STATUSES[STATUSES["OK"] = 200] = "OK";
    STATUSES[STATUSES["CREATED"] = 201] = "CREATED";
    STATUSES[STATUSES["ACCEPTED"] = 202] = "ACCEPTED";
    STATUSES[STATUSES["NON_AUTHORITATIVE_INFORMATION"] = 203] = "NON_AUTHORITATIVE_INFORMATION";
    STATUSES[STATUSES["NO_CONTENT"] = 204] = "NO_CONTENT";
    STATUSES[STATUSES["RESET_CONTENT"] = 205] = "RESET_CONTENT";
    STATUSES[STATUSES["PARTIAL_CONTENT"] = 206] = "PARTIAL_CONTENT";
    STATUSES[STATUSES["MULTI_STATUS"] = 207] = "MULTI_STATUS";
    STATUSES[STATUSES["ALREADY_REPORTED"] = 208] = "ALREADY_REPORTED";
    STATUSES[STATUSES["TRANSFORMATION_APPLIED"] = 214] = "TRANSFORMATION_APPLIED";
    STATUSES[STATUSES["IM_USED"] = 226] = "IM_USED";
    STATUSES[STATUSES["MISCELLANEOUS_PERSISTENT_WARNING"] = 299] = "MISCELLANEOUS_PERSISTENT_WARNING";
    STATUSES[STATUSES["MULTIPLE_CHOICES"] = 300] = "MULTIPLE_CHOICES";
    STATUSES[STATUSES["MOVED_PERMANENTLY"] = 301] = "MOVED_PERMANENTLY";
    STATUSES[STATUSES["FOUND"] = 302] = "FOUND";
    STATUSES[STATUSES["SEE_OTHER"] = 303] = "SEE_OTHER";
    STATUSES[STATUSES["NOT_MODIFIED"] = 304] = "NOT_MODIFIED";
    STATUSES[STATUSES["USE_PROXY"] = 305] = "USE_PROXY";
    STATUSES[STATUSES["SWITCH_PROXY"] = 306] = "SWITCH_PROXY";
    STATUSES[STATUSES["TEMPORARY_REDIRECT"] = 307] = "TEMPORARY_REDIRECT";
    STATUSES[STATUSES["PERMANENT_REDIRECT"] = 308] = "PERMANENT_REDIRECT";
    STATUSES[STATUSES["BAD_REQUEST"] = 400] = "BAD_REQUEST";
    STATUSES[STATUSES["UNAUTHORIZED"] = 401] = "UNAUTHORIZED";
    STATUSES[STATUSES["PAYMENT_REQUIRED"] = 402] = "PAYMENT_REQUIRED";
    STATUSES[STATUSES["FORBIDDEN"] = 403] = "FORBIDDEN";
    STATUSES[STATUSES["NOT_FOUND"] = 404] = "NOT_FOUND";
    STATUSES[STATUSES["METHOD_NOT_ALLOWED"] = 405] = "METHOD_NOT_ALLOWED";
    STATUSES[STATUSES["NOT_ACCEPTABLE"] = 406] = "NOT_ACCEPTABLE";
    STATUSES[STATUSES["PROXY_AUTHENTICATION_REQUIRED"] = 407] = "PROXY_AUTHENTICATION_REQUIRED";
    STATUSES[STATUSES["REQUEST_TIMEOUT"] = 408] = "REQUEST_TIMEOUT";
    STATUSES[STATUSES["CONFLICT"] = 409] = "CONFLICT";
    STATUSES[STATUSES["GONE"] = 410] = "GONE";
    STATUSES[STATUSES["LENGTH_REQUIRED"] = 411] = "LENGTH_REQUIRED";
    STATUSES[STATUSES["PRECONDITION_FAILED"] = 412] = "PRECONDITION_FAILED";
    STATUSES[STATUSES["PAYLOAD_TOO_LARGE"] = 413] = "PAYLOAD_TOO_LARGE";
    STATUSES[STATUSES["URI_TOO_LONG"] = 414] = "URI_TOO_LONG";
    STATUSES[STATUSES["UNSUPPORTED_MEDIA_TYPE"] = 415] = "UNSUPPORTED_MEDIA_TYPE";
    STATUSES[STATUSES["RANGE_NOT_SATISFIABLE"] = 416] = "RANGE_NOT_SATISFIABLE";
    STATUSES[STATUSES["EXPECTATION_FAILED"] = 417] = "EXPECTATION_FAILED";
    STATUSES[STATUSES["IM_A_TEAPOT"] = 418] = "IM_A_TEAPOT";
    STATUSES[STATUSES["PAGE_EXPIRED"] = 419] = "PAGE_EXPIRED";
    STATUSES[STATUSES["ENHANCE_YOUR_CALM"] = 420] = "ENHANCE_YOUR_CALM";
    STATUSES[STATUSES["MISDIRECTED_REQUEST"] = 421] = "MISDIRECTED_REQUEST";
    STATUSES[STATUSES["UNPROCESSABLE_ENTITY"] = 422] = "UNPROCESSABLE_ENTITY";
    STATUSES[STATUSES["LOCKED"] = 423] = "LOCKED";
    STATUSES[STATUSES["FAILED_DEPENDENCY"] = 424] = "FAILED_DEPENDENCY";
    STATUSES[STATUSES["TOO_EARLY"] = 425] = "TOO_EARLY";
    STATUSES[STATUSES["UPGRADE_REQUIRED"] = 426] = "UPGRADE_REQUIRED";
    STATUSES[STATUSES["PRECONDITION_REQUIRED"] = 428] = "PRECONDITION_REQUIRED";
    STATUSES[STATUSES["TOO_MANY_REQUESTS"] = 429] = "TOO_MANY_REQUESTS";
    STATUSES[STATUSES["REQUEST_HEADER_FIELDS_TOO_LARGE_UNOFFICIAL"] = 430] = "REQUEST_HEADER_FIELDS_TOO_LARGE_UNOFFICIAL";
    STATUSES[STATUSES["REQUEST_HEADER_FIELDS_TOO_LARGE"] = 431] = "REQUEST_HEADER_FIELDS_TOO_LARGE";
    STATUSES[STATUSES["LOGIN_TIMEOUT"] = 440] = "LOGIN_TIMEOUT";
    STATUSES[STATUSES["NO_RESPONSE"] = 444] = "NO_RESPONSE";
    STATUSES[STATUSES["RETRY_WITH"] = 449] = "RETRY_WITH";
    STATUSES[STATUSES["BLOCKED_BY_PARENTAL_CONTROL"] = 450] = "BLOCKED_BY_PARENTAL_CONTROL";
    STATUSES[STATUSES["UNAVAILABLE_FOR_LEGAL_REASONS"] = 451] = "UNAVAILABLE_FOR_LEGAL_REASONS";
    STATUSES[STATUSES["CLIENT_CLOSED_LOAD_BALANCED_REQUEST"] = 460] = "CLIENT_CLOSED_LOAD_BALANCED_REQUEST";
    STATUSES[STATUSES["INVALID_X_FORWARDED_FOR"] = 463] = "INVALID_X_FORWARDED_FOR";
    STATUSES[STATUSES["REQUEST_HEADER_TOO_LARGE"] = 494] = "REQUEST_HEADER_TOO_LARGE";
    STATUSES[STATUSES["SSL_CERTIFICATE_ERROR"] = 495] = "SSL_CERTIFICATE_ERROR";
    STATUSES[STATUSES["SSL_CERTIFICATE_REQUIRED"] = 496] = "SSL_CERTIFICATE_REQUIRED";
    STATUSES[STATUSES["HTTP_REQUEST_SENT_TO_HTTPS_PORT"] = 497] = "HTTP_REQUEST_SENT_TO_HTTPS_PORT";
    STATUSES[STATUSES["INVALID_TOKEN"] = 498] = "INVALID_TOKEN";
    STATUSES[STATUSES["CLIENT_CLOSED_REQUEST"] = 499] = "CLIENT_CLOSED_REQUEST";
    STATUSES[STATUSES["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
    STATUSES[STATUSES["NOT_IMPLEMENTED"] = 501] = "NOT_IMPLEMENTED";
    STATUSES[STATUSES["BAD_GATEWAY"] = 502] = "BAD_GATEWAY";
    STATUSES[STATUSES["SERVICE_UNAVAILABLE"] = 503] = "SERVICE_UNAVAILABLE";
    STATUSES[STATUSES["GATEWAY_TIMEOUT"] = 504] = "GATEWAY_TIMEOUT";
    STATUSES[STATUSES["HTTP_VERSION_NOT_SUPPORTED"] = 505] = "HTTP_VERSION_NOT_SUPPORTED";
    STATUSES[STATUSES["VARIANT_ALSO_NEGOTIATES"] = 506] = "VARIANT_ALSO_NEGOTIATES";
    STATUSES[STATUSES["INSUFFICIENT_STORAGE"] = 507] = "INSUFFICIENT_STORAGE";
    STATUSES[STATUSES["LOOP_DETECTED"] = 508] = "LOOP_DETECTED";
    STATUSES[STATUSES["BANDWIDTH_LIMIT_EXCEEDED"] = 509] = "BANDWIDTH_LIMIT_EXCEEDED";
    STATUSES[STATUSES["NOT_EXTENDED"] = 510] = "NOT_EXTENDED";
    STATUSES[STATUSES["NETWORK_AUTHENTICATION_REQUIRED"] = 511] = "NETWORK_AUTHENTICATION_REQUIRED";
    STATUSES[STATUSES["WEB_SERVER_UNKNOWN_ERROR"] = 520] = "WEB_SERVER_UNKNOWN_ERROR";
    STATUSES[STATUSES["WEB_SERVER_IS_DOWN"] = 521] = "WEB_SERVER_IS_DOWN";
    STATUSES[STATUSES["CONNECTION_TIMEOUT"] = 522] = "CONNECTION_TIMEOUT";
    STATUSES[STATUSES["ORIGIN_IS_UNREACHABLE"] = 523] = "ORIGIN_IS_UNREACHABLE";
    STATUSES[STATUSES["TIMEOUT_OCCURED"] = 524] = "TIMEOUT_OCCURED";
    STATUSES[STATUSES["SSL_HANDSHAKE_FAILED"] = 525] = "SSL_HANDSHAKE_FAILED";
    STATUSES[STATUSES["INVALID_SSL_CERTIFICATE"] = 526] = "INVALID_SSL_CERTIFICATE";
    STATUSES[STATUSES["RAILGUN_ERROR"] = 527] = "RAILGUN_ERROR";
    STATUSES[STATUSES["SITE_IS_OVERLOADED"] = 529] = "SITE_IS_OVERLOADED";
    STATUSES[STATUSES["SITE_IS_FROZEN"] = 530] = "SITE_IS_FROZEN";
    STATUSES[STATUSES["IDENTITY_PROVIDER_AUTHENTICATION_ERROR"] = 561] = "IDENTITY_PROVIDER_AUTHENTICATION_ERROR";
    STATUSES[STATUSES["NETWORK_READ_TIMEOUT"] = 598] = "NETWORK_READ_TIMEOUT";
    STATUSES[STATUSES["NETWORK_CONNECT_TIMEOUT"] = 599] = "NETWORK_CONNECT_TIMEOUT";
})(STATUSES = exports.STATUSES || (exports.STATUSES = {}));
exports.STATUSES_HTTP = [
    STATUSES.CONTINUE,
    STATUSES.SWITCHING_PROTOCOLS,
    STATUSES.PROCESSING,
    STATUSES.EARLY_HINTS,
    STATUSES.RESPONSE_IS_STALE,
    STATUSES.REVALIDATION_FAILED,
    STATUSES.DISCONNECTED_OPERATION,
    STATUSES.HEURISTIC_EXPIRATION,
    STATUSES.MISCELLANEOUS_WARNING,
    STATUSES.OK,
    STATUSES.CREATED,
    STATUSES.ACCEPTED,
    STATUSES.NON_AUTHORITATIVE_INFORMATION,
    STATUSES.NO_CONTENT,
    STATUSES.RESET_CONTENT,
    STATUSES.PARTIAL_CONTENT,
    STATUSES.MULTI_STATUS,
    STATUSES.ALREADY_REPORTED,
    STATUSES.TRANSFORMATION_APPLIED,
    STATUSES.IM_USED,
    STATUSES.MISCELLANEOUS_PERSISTENT_WARNING,
    STATUSES.MULTIPLE_CHOICES,
    STATUSES.MOVED_PERMANENTLY,
    STATUSES.FOUND,
    STATUSES.SEE_OTHER,
    STATUSES.NOT_MODIFIED,
    STATUSES.USE_PROXY,
    STATUSES.SWITCH_PROXY,
    STATUSES.TEMPORARY_REDIRECT,
    STATUSES.PERMANENT_REDIRECT,
    STATUSES.BAD_REQUEST,
    STATUSES.UNAUTHORIZED,
    STATUSES.PAYMENT_REQUIRED,
    STATUSES.FORBIDDEN,
    STATUSES.NOT_FOUND,
    STATUSES.METHOD_NOT_ALLOWED,
    STATUSES.NOT_ACCEPTABLE,
    STATUSES.PROXY_AUTHENTICATION_REQUIRED,
    STATUSES.REQUEST_TIMEOUT,
    STATUSES.CONFLICT,
    STATUSES.GONE,
    STATUSES.LENGTH_REQUIRED,
    STATUSES.PRECONDITION_FAILED,
    STATUSES.PAYLOAD_TOO_LARGE,
    STATUSES.URI_TOO_LONG,
    STATUSES.UNSUPPORTED_MEDIA_TYPE,
    STATUSES.RANGE_NOT_SATISFIABLE,
    STATUSES.EXPECTATION_FAILED,
    STATUSES.IM_A_TEAPOT,
    STATUSES.PAGE_EXPIRED,
    STATUSES.ENHANCE_YOUR_CALM,
    STATUSES.MISDIRECTED_REQUEST,
    STATUSES.UNPROCESSABLE_ENTITY,
    STATUSES.LOCKED,
    STATUSES.FAILED_DEPENDENCY,
    STATUSES.TOO_EARLY,
    STATUSES.UPGRADE_REQUIRED,
    STATUSES.PRECONDITION_REQUIRED,
    STATUSES.TOO_MANY_REQUESTS,
    STATUSES.REQUEST_HEADER_FIELDS_TOO_LARGE_UNOFFICIAL,
    STATUSES.REQUEST_HEADER_FIELDS_TOO_LARGE,
    STATUSES.LOGIN_TIMEOUT,
    STATUSES.NO_RESPONSE,
    STATUSES.RETRY_WITH,
    STATUSES.BLOCKED_BY_PARENTAL_CONTROL,
    STATUSES.UNAVAILABLE_FOR_LEGAL_REASONS,
    STATUSES.CLIENT_CLOSED_LOAD_BALANCED_REQUEST,
    STATUSES.INVALID_X_FORWARDED_FOR,
    STATUSES.REQUEST_HEADER_TOO_LARGE,
    STATUSES.SSL_CERTIFICATE_ERROR,
    STATUSES.SSL_CERTIFICATE_REQUIRED,
    STATUSES.HTTP_REQUEST_SENT_TO_HTTPS_PORT,
    STATUSES.INVALID_TOKEN,
    STATUSES.CLIENT_CLOSED_REQUEST,
    STATUSES.INTERNAL_SERVER_ERROR,
    STATUSES.NOT_IMPLEMENTED,
    STATUSES.BAD_GATEWAY,
    STATUSES.SERVICE_UNAVAILABLE,
    STATUSES.GATEWAY_TIMEOUT,
    STATUSES.HTTP_VERSION_NOT_SUPPORTED,
    STATUSES.VARIANT_ALSO_NEGOTIATES,
    STATUSES.INSUFFICIENT_STORAGE,
    STATUSES.LOOP_DETECTED,
    STATUSES.BANDWIDTH_LIMIT_EXCEEDED,
    STATUSES.NOT_EXTENDED,
    STATUSES.NETWORK_AUTHENTICATION_REQUIRED,
    STATUSES.WEB_SERVER_UNKNOWN_ERROR,
    STATUSES.WEB_SERVER_IS_DOWN,
    STATUSES.CONNECTION_TIMEOUT,
    STATUSES.ORIGIN_IS_UNREACHABLE,
    STATUSES.TIMEOUT_OCCURED,
    STATUSES.SSL_HANDSHAKE_FAILED,
    STATUSES.INVALID_SSL_CERTIFICATE,
    STATUSES.RAILGUN_ERROR,
    STATUSES.SITE_IS_OVERLOADED,
    STATUSES.SITE_IS_FROZEN,
    STATUSES.IDENTITY_PROVIDER_AUTHENTICATION_ERROR,
    STATUSES.NETWORK_READ_TIMEOUT,
    STATUSES.NETWORK_CONNECT_TIMEOUT,
];
var FINISH;
(function (FINISH) {
    FINISH[FINISH["SAFE"] = 0] = "SAFE";
    FINISH[FINISH["SAFE_WITH_CB"] = 1] = "SAFE_WITH_CB";
    FINISH[FINISH["UNSAFE"] = 2] = "UNSAFE";
})(FINISH = exports.FINISH || (exports.FINISH = {}));
exports.ALPHA = [];
for (let i = 'A'.charCodeAt(0); i <= 'Z'.charCodeAt(0); i++) {
    // Upper case
    exports.ALPHA.push(String.fromCharCode(i));
    // Lower case
    exports.ALPHA.push(String.fromCharCode(i + 0x20));
}
exports.NUM_MAP = {
    0: 0, 1: 1, 2: 2, 3: 3, 4: 4,
    5: 5, 6: 6, 7: 7, 8: 8, 9: 9,
};
exports.HEX_MAP = {
    0: 0, 1: 1, 2: 2, 3: 3, 4: 4,
    5: 5, 6: 6, 7: 7, 8: 8, 9: 9,
    A: 0XA, B: 0XB, C: 0XC, D: 0XD, E: 0XE, F: 0XF,
    a: 0xa, b: 0xb, c: 0xc, d: 0xd, e: 0xe, f: 0xf,
};
exports.NUM = [
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
];
exports.ALPHANUM = exports.ALPHA.concat(exports.NUM);
exports.MARK = ['-', '_', '.', '!', '~', '*', '\'', '(', ')'];
exports.USERINFO_CHARS = exports.ALPHANUM
    .concat(exports.MARK)
    .concat(['%', ';', ':', '&', '=', '+', '$', ',']);
// TODO(indutny): use RFC
exports.URL_CHAR = [
    '!', '"', '$', '%', '&', '\'',
    '(', ')', '*', '+', ',', '-', '.', '/',
    ':', ';', '<', '=', '>',
    '@', '[', '\\', ']', '^', '_',
    '`',
    '{', '|', '}', '~',
].concat(exports.ALPHANUM);
exports.HEX = exports.NUM.concat(['a', 'b', 'c', 'd', 'e', 'f', 'A', 'B', 'C', 'D', 'E', 'F']);
/* Tokens as defined by rfc 2616. Also lowercases them.
 *        token       = 1*<any CHAR except CTLs or separators>
 *     separators     = "(" | ")" | "<" | ">" | "@"
 *                    | "," | ";" | ":" | "\" | <">
 *                    | "/" | "[" | "]" | "?" | "="
 *                    | "{" | "}" | SP | HT
 */
exports.TOKEN = [
    '!', '#', '$', '%', '&', '\'',
    '*', '+', '-', '.',
    '^', '_', '`',
    '|', '~',
].concat(exports.ALPHANUM);
/*
 * Verify that a char is a valid visible (printable) US-ASCII
 * character or %x80-FF
 */
exports.HEADER_CHARS = ['\t'];
for (let i = 32; i <= 255; i++) {
    if (i !== 127) {
        exports.HEADER_CHARS.push(i);
    }
}
// ',' = \x44
exports.CONNECTION_TOKEN_CHARS = exports.HEADER_CHARS.filter((c) => c !== 44);
exports.QUOTED_STRING = ['\t', ' '];
for (let i = 0x21; i <= 0xff; i++) {
    if (i !== 0x22 && i !== 0x5c) { // All characters in ASCII except \ and "
        exports.QUOTED_STRING.push(i);
    }
}
exports.HTAB_SP_VCHAR_OBS_TEXT = ['\t', ' '];
// VCHAR: https://tools.ietf.org/html/rfc5234#appendix-B.1
for (let i = 0x21; i <= 0x7E; i++) {
    exports.HTAB_SP_VCHAR_OBS_TEXT.push(i);
}
// OBS_TEXT: https://datatracker.ietf.org/doc/html/rfc9110#name-collected-abnf
for (let i = 0x80; i <= 0xff; i++) {
    exports.HTAB_SP_VCHAR_OBS_TEXT.push(i);
}
exports.MAJOR = exports.NUM_MAP;
exports.MINOR = exports.MAJOR;
var HEADER_STATE;
(function (HEADER_STATE) {
    HEADER_STATE[HEADER_STATE["GENERAL"] = 0] = "GENERAL";
    HEADER_STATE[HEADER_STATE["CONNECTION"] = 1] = "CONNECTION";
    HEADER_STATE[HEADER_STATE["CONTENT_LENGTH"] = 2] = "CONTENT_LENGTH";
    HEADER_STATE[HEADER_STATE["TRANSFER_ENCODING"] = 3] = "TRANSFER_ENCODING";
    HEADER_STATE[HEADER_STATE["UPGRADE"] = 4] = "UPGRADE";
    HEADER_STATE[HEADER_STATE["CONNECTION_KEEP_ALIVE"] = 5] = "CONNECTION_KEEP_ALIVE";
    HEADER_STATE[HEADER_STATE["CONNECTION_CLOSE"] = 6] = "CONNECTION_CLOSE";
    HEADER_STATE[HEADER_STATE["CONNECTION_UPGRADE"] = 7] = "CONNECTION_UPGRADE";
    HEADER_STATE[HEADER_STATE["TRANSFER_ENCODING_CHUNKED"] = 8] = "TRANSFER_ENCODING_CHUNKED";
})(HEADER_STATE = exports.HEADER_STATE || (exports.HEADER_STATE = {}));
exports.SPECIAL_HEADERS = {
    'connection': HEADER_STATE.CONNECTION,
    'content-length': HEADER_STATE.CONTENT_LENGTH,
    'proxy-connection': HEADER_STATE.CONNECTION,
    'transfer-encoding': HEADER_STATE.TRANSFER_ENCODING,
    'upgrade': HEADER_STATE.UPGRADE,
};
//# sourceMappingURL=constants.js.map